package io.github.beradq.adybubbles

class Bubbles(val items: MutableList<String>, val config: BubbleConfig) : Cloneable {
    private val state = BubbleState(0, if (config.mode == ChatPopupMode.ONCE) -1L else 0)
    private var restDelay = config.restDelayRange.random()
    private var period = config.period.random()

    operator fun get(index: Int): String {
        return items[index]
    }

    data class BubbleConfig(
        val mode: ChatPopupMode,
        val _restDelayRange: LongRange? = null,
        val _period: LongRange? = null
    ) {
        val period: LongRange get() = _period ?: Config.bubblesConfig.chat.period
        val restDelayRange: LongRange get() = _restDelayRange ?: Config.bubblesConfig.chat.restDelay
    }

    data class BubbleState(
        var index: Int,
        var timeCount: Long
    ) : Cloneable

    fun next(): String? {
        if (config.mode == ChatPopupMode.RANDOM) return items.random()
        if (items.isEmpty()) return null
        val item = items[state.index]
        state.index++
        if (state.index >= items.size && config.mode == ChatPopupMode.LOOP) {
            state.index = 0
        }
        return item
    }

    fun tick(uuid: String) {
        if (state.timeCount == -1L) return
        state.timeCount += 1
        if (state.timeCount == period) {
            popup(uuid, next() ?: return)
        }
        if (state.timeCount >= period) {
            if (state.index != 0 || config.mode != ChatPopupMode.LOOP) {
                state.timeCount = if (config.mode == ChatPopupMode.ONCE) -1 else 0
                period = config.period.random()
            }
            if (state.timeCount >= period + restDelay) {
                state.timeCount = 0
                period = config.period.random()
                restDelay = config.restDelayRange.random()
            }
        }
    }

    private fun popup(uuid: String, text: String) {
        if (Config.bubblesConfig.chat.lifetime != null) {
            BubblesBundle.popupTick(uuid, text, Config.bubblesConfig.chat.lifetime!!)
        } else {
            BubblesBundle.popup(uuid, text)
        }
        BubblesBundle.limit(uuid, Config.bubblesConfig.chat.limit)
    }

    fun resetState() {
        state.index = 0
        state.timeCount = 0
        restDelay = config.restDelayRange.random()
        period = config.period.random()
    }

    public override fun clone(): Bubbles {
        val bubbles = Bubbles(items.toMutableList(), config.copy())
        bubbles.state.index = state.index
        bubbles.state.timeCount = state.timeCount
        return bubbles
    }
}
