package io.github.beradq.adybubbles

import ink.ptms.adyeshach.core.AdyeshachHologram
import io.github.beradq.adybubbles.AdyBubbles.adyApi
import io.github.beradq.adybubbles.AdyBubbles.entityFinder
import io.github.beradq.adybubbles.AdyBubbles.hologramHandler
import taboolib.common.LifeCycle
import taboolib.common.platform.Awake
import taboolib.common.platform.function.submit


object BubblesBundle {
    private val map: MutableMap<String, MutableList<AdyeshachHologram>> = mutableMapOf()
    operator fun get(key: String): List<AdyeshachHologram>? {
        return map[key]
    }

    fun popup(npcId: String, text: String): AdyeshachHologram {
        val adyEntity = entityFinder.getEntityFromUniqueId(npcId) ?: error("Entity not found: $npcId")
        val hologramList = map.getOrPut(npcId) {
            mutableListOf()
        }
        val hologram = hologramHandler.createHologram(
            adyEntity.getLocation().add(0.0, adyEntity.entitySize.height, 0.0), listOf(text)
        )
        hologramList.add(hologram)
        updateLocation(npcId)
        return hologram
    }

    fun popupTick(npcId: String, text: String, lifetime: Long) {
        val hologram = popup(npcId, text)
        submit(delay = lifetime) {
            hologram.remove()
            map[text]?.remove(hologram)
            this.cancel()
        }
    }

    fun clear(npcId: String) {
        map[npcId]?.forEach { it.remove() }
        map[npcId]?.clear()
    }

    fun deleteFirst(npcId: String) {
        map[npcId]?.removeFirst()?.remove()
    }

    fun limit(npcId: String, limit: Int) {
        if (!map.containsKey(npcId)) return
        while (map[npcId]!!.size > limit) {
            deleteFirst(npcId)
        }
    }

    fun updateLocation(npcId: String) {
        val adyEntity = entityFinder.getEntityFromUniqueId(npcId) ?: error("Entity not found: $npcId")
        val hologramList = map.getOrPut(npcId) { mutableListOf() }
        hologramList.reversed().withIndex().forEach {
            it.value.teleport(
                adyEntity.getLocation().add(
                    0.0,
                    adyEntity.entitySize.height + Config.bubblesConfig.offset + it.index * Config.bubblesConfig.lineHeight,
                    0.0
                )
            )
        }
    }

    @Awake(LifeCycle.ACTIVE)
    private fun enable() {
        adyApi.getEventBus().prepareTeleport { e ->
            val hologram = this[e.entity.uniqueId] ?: return@prepareTeleport true
            hologram.reversed().withIndex().forEach {
                it.value.teleport(
                    e.location.clone().add(
                        0.0,
                        e.entity.entitySize.height + Config.bubblesConfig.offset + it.index * Config.bubblesConfig.lineHeight,
                        0.0
                    )
                )
            }
            true
        }
    }
}