package io.github.beradq.adybubbles

import org.bukkit.command.CommandSender
import org.bukkit.entity.Player
import taboolib.common.platform.command.CommandBody
import taboolib.common.platform.command.CommandHeader
import taboolib.common.platform.command.subCommand
import io.github.beradq.adybubbles.AdyBubbles.entityFinder
import taboolib.common.platform.ProxyCommandSender
import taboolib.common.platform.command.bool
import taboolib.common.platform.function.adaptCommandSender
import taboolib.common.platform.function.adaptPlayer
import taboolib.module.chat.component
import taboolib.module.nms.*

private fun clearMessageScreen(player: ProxyCommandSender) {
    for (i in 1..5) player.sendMessage(" ")
}

private fun edd(uuid: String, cmd: String): String {
    return "/bubbles-chat editdata true $uuid $cmd"
}

private fun String.replaceBracketForSafety(): String {
    return this.replace("[", "\\[").replace("]", "\\]")
}

private fun sendEditMessage(player: ProxyCommandSender, uuid: String, title: String) {
    clearMessageScreen(player)
    val bubbles = TraitBubblesChat.getOrCreateBubbles(uuid)
    player.sendMessage("当前正在编辑: $title 的弹出泡泡")
    player.sendMessage(" ")
    player.sendMessage(" ")
    player.sendMessage(" ")
    "[§b\\[ 弹出内容 \\]§f](cmd=${edd(uuid, "items")})".component().sendTo(player)
    player.sendMessage(" ")
    player.sendMessage(" ")
    player.sendMessage("弹出模式:")
    when (bubbles.config.mode) {
        ChatPopupMode.LOOP ->
            "[§a\\[循环\\]§f](b;u) [§b\\[随机\\]§f](cmd=${edd(uuid, "mode random")}) [§b\\[单次\\]§f](cmd=${
                edd(
                    uuid,
                    "mode once"
                )
            })"
                .component().sendTo(player)

        ChatPopupMode.RANDOM ->
            "[§b\\[循环\\]§f](cmd=${edd(uuid, "mode loop")}) [§a\\[随机\\]§f](b;u) [§b\\[单次\\]§f](cmd=${
                edd(
                    uuid,
                    "mode once"
                )
            })"
                .component().sendTo(player)

        ChatPopupMode.ONCE ->
            "[§b\\[循环\\]§f](cmd=${edd(uuid, "mode loop")}) [§b\\[随机\\]§f](cmd=${
                edd(
                    uuid,
                    "mode random"
                )
            }) [§a\\[单次\\]§f](b;u)"
                .component().sendTo(player)
    }
    player.sendMessage(" ")
    player.sendMessage(" ")
    if (bubbles.config._period != null) {
        "弹出间隔: [§b\\[ ${bubbles.config._period.toMString().replaceBracketForSafety()} \\]§f](cmd=${
            edd(
                uuid,
                "period"
            )
        })"
            .component().sendTo(player)
    } else {
        "弹出间隔: [§b\\[ 默认值 \\]§f](cmd=${edd(uuid, "period")})"
            .component().sendTo(player)
    }
    if (bubbles.config._restDelayRange != null) {
        "每轮间隔: [§b\\[ ${bubbles.config._restDelayRange.toMString().replaceBracketForSafety()} \\]§f](cmd=${
            edd(
                uuid,
                "rest-delay"
            )
        })"
            .component().sendTo(player)
    } else {
        "每轮间隔: [§b\\[ 默认值 \\]§f](cmd=${edd(uuid, "rest-delay")})"
            .component().sendTo(player)
    }
    player.sendMessage(" ")
    player.sendMessage(" ")
    player.sendMessage(" ")
    player.sendMessage("蓝色选项可点击编辑")
    player.sendMessage("请展开聊天栏编辑")
}

@CommandHeader("bubbles-chat")
@Suppress("unused")
object BubblesChatCommand {

    @CommandBody
    @Suppress("unused")
    val play = subCommand {
        dynamic("UUID") {
            suggestEntityListUnique()
            execute<CommandSender> { _, context, _ ->
                val uuid = context["UUID"]
                TraitBubblesChat.startOnce(uuid)
            }
        }
    }

    @CommandBody
    @Suppress("unused")
    val editdata = subCommand {
        bool("RECALL") {
            dynamic("UUID") {
                suggestEntityListUnique()
                dynamic("FIELD") {
                    suggestion<CommandSender>(uncheck = false) { _, _ ->
                        listOf("items", "mode", "period", "rest-delay")
                    }
                    execute<CommandSender> { sender, context, _ ->
                        val field = context["FIELD"]
                        val recall = context.bool("RECALL")
                        val uuid = context["UUID"]
                        val bubbles = TraitBubblesChat.getOrCreateBubbles(uuid)
                        val npc =
                            entityFinder.getEntityFromUniqueId(uuid) ?: throw Exception("Entity not found")
                        when (field) {
                            "items" -> {
                                TraitBubblesChat.edit(sender as Player, npc)
                                sender.sendTitle("请编辑书", null, 1, 20, 1)
                            }

                            "period" -> {
                                (sender as Player).inputSign(
                                    arrayOf(
                                        bubbles.config._period?.toMString() ?: "",
                                        "请在第一行输入数字或",
                                        "区间代表范围内随机"
                                    )
                                ) {
                                    val content = it.first().trim()
                                    if (content.isEmpty()) {
                                        TraitBubblesChat.setPeriod(uuid, null)
                                    } else {
                                        val range = content.toLongRange()
                                        TraitBubblesChat.setPeriod(uuid, range)
                                    }
                                    if (recall) sendEditMessage(adaptCommandSender(sender), uuid, npc.id)
                                }
                            }

                            "rest-delay" -> {
                                (sender as Player).inputSign(
                                    arrayOf(
                                        bubbles.config._restDelayRange?.toMString() ?: "",
                                        "请在第一行输入数字或",
                                        "区间代表范围内随机"
                                    )
                                ) {
                                    val content = it.first().trim()
                                    if (content.isEmpty()) {
                                        TraitBubblesChat.setRestDelay(uuid, null)
                                    } else {
                                        val range = content.toLongRange()
                                        TraitBubblesChat.setRestDelay(uuid, range)
                                    }
                                    if (recall) sendEditMessage(adaptCommandSender(sender), uuid, npc.id)
                                }
                            }

                            else -> {
                                sender.sendMessage("缺少参数")
                                return@execute
                            }
                        }
                    }
                    dynamic("VALUE") {
                        execute<CommandSender> { sender, context, _ ->
                            val field = context["FIELD"]
                            val recall = context.bool("RECALL")
                            val value = context["VALUE"]
                            val uuid = context["UUID"]
                            val npc = entityFinder.getEntityFromUniqueId(uuid) ?: throw Exception("Entity not found")
                            when (field) {
                                "mode" -> {
                                    val mode = ChatPopupMode.fromString(value)
                                    TraitBubblesChat.setMode(uuid, mode)
                                    if (recall) sendEditMessage(adaptCommandSender(sender), uuid, npc.id)
                                }

                                else -> {
                                    sender.sendMessage("Unknown field: $field")
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    @CommandBody
    @Suppress("unused")
    val edit = subCommand {
        dynamic("NPC_ID") {
            suggestEntityList()
            execute<CommandSender> { sender, context, _ ->
                val npcs = entityFinder.getEntitiesFromId(context["NPC_ID"])
                if (npcs.size == 1) {
                    val uuid = npcs[0].uniqueId
                    sendEditMessage(adaptPlayer(sender as Player), uuid, npcs[0].id)
                } else {
                    sender.sendMessage("找到了${npcs.size}个NPC:")
                    for (npc in npcs) {
                        "[${npc.uniqueId}](cmd=/bubbles-chat edit2 ${npc.uniqueId})".component().sendTo(
                            adaptCommandSender(sender)
                        )
                    }
                }
            }
        }
    }

    @CommandBody
    @Suppress("unused")
    val edit2 = subCommand {
        dynamic("UUID") {
            suggestEntityListUnique()
            execute<CommandSender> { sender, context, _ ->
                val uuid = context["UUID"]
                val npc = entityFinder.getEntityFromUniqueId(uuid)!!
                sendEditMessage(adaptPlayer(sender as Player), uuid, npc.id)
            }
        }
    }
}