package io.github.beradq.adybubbles

import taboolib.common.LifeCycle
import taboolib.common.platform.Awake
import taboolib.module.configuration.Config
import taboolib.module.configuration.ConfigFile

object Config {
    @Config("config.yml")
    lateinit var configFile: ConfigFile
    lateinit var bubblesConfig: BubblesConfig

    class BubblesChatConfig(
        val period: LongRange,
        val restDelay: LongRange,
        val limit: Int,
        val lifetime: Long?
    )

    class BubblesConfig(
        val offset: Double,
        val lineHeight: Double,
        val chat: BubblesChatConfig
    )

    @Awake(LifeCycle.ENABLE)
    fun reload() {
        configFile.reload()
        val offset = configFile.getDouble("offset", .5)
        val lineHeight = configFile.getDouble("line-height", 0.5)
        val chat = configFile.getConfigurationSection("chat")
        val period = chat?.getString("period")?.toLongRange() ?: (20L * 3)..(20 * 3)
        val limit = chat?.getInt("limit", 3) ?: 3
        val restDelayRange = chat?.getString("rest-delay")?.toLongRange() ?: 0L..0
        val lifetime = chat?.getLong("lifetime")
        bubblesConfig = BubblesConfig(offset, lineHeight, BubblesChatConfig(period, restDelayRange, limit, lifetime))
    }
}