package io.github.beradq.adybubbles

import ink.ptms.adyeshach.core.entity.EntityInstance
import ink.ptms.adyeshach.core.event.AdyeshachEntityVisibleEvent
import ink.ptms.adyeshach.impl.entity.trait.Trait
import ink.ptms.adyeshach.impl.util.Inputs.inputBook
import org.bukkit.entity.Player
import taboolib.common.platform.Schedule
import taboolib.common.platform.event.EventPriority
import taboolib.common.platform.event.SubscribeEvent
import taboolib.module.chat.uncolored
import java.util.concurrent.CompletableFuture

object TraitBubblesChat : Trait() {
    private val bubbles = mutableMapOf<String, Bubbles>()
    private val visibleEntity = mutableListOf<String>()

    fun getBubbles(uuid: String): Bubbles? {
        if (!data.contains(uuid)) return null
        val conf = data.getConfigurationSection(uuid)!!
        val items = conf.getStringList("items")
        val mode = conf.getString("mode")?.let { ChatPopupMode.fromString(it) } ?: ChatPopupMode.LOOP
        val period = conf.getString("period")?.toLongRange()
        val restDelay = conf.getString("rest-delay")?.toLongRange()
        return Bubbles(items.toMutableList(), Bubbles.BubbleConfig(mode, restDelay, period))
    }

    @Suppress("unused")
    fun setBubbles(uuid: String, bubbles: Bubbles) {
        data[uuid] = mapOf(
            "items" to bubbles.items,
            "mode" to bubbles.config.mode.toString(),
            "period" to bubbles.config._period?.toMString(),
            "rest-delay" to bubbles.config._restDelayRange?.toMString()
        )
        uuid pipe this::update
    }

    fun getOrCreateBubbles(uuid: String): Bubbles {
        val bubbles = bubbles.getOrPut(uuid) { Bubbles(mutableListOf(), Bubbles.BubbleConfig(ChatPopupMode.LOOP)) }
        update(uuid)
        return bubbles
    }

    override fun edit(player: Player, entityInstance: EntityInstance): CompletableFuture<Void> {
        val future = CompletableFuture<Void>()
        val uuid = entityInstance.uniqueId
        val content = getOrCreateBubbles(uuid).items
        player.inputBook(content) {
            future.complete(null)
            if (it.all { s -> s.isBlank() }) {
                data[uuid] = null
                uuid also bubbles::remove also BubblesBundle::clear
            } else {
                getConfiguration(uuid)["items"] = it.uncolored()
            }
            uuid pipe this::update
        }
        return future
    }

    fun setMode(uuid: String, mode: ChatPopupMode) {
        getConfiguration(uuid)["mode"] = mode.toString()
        uuid pipe this::update
    }

    fun setPeriod(uuid: String, period: LongRange?) {
        getConfiguration(uuid)["period"] = period?.toMString()
        uuid pipe this::update
    }

    fun setRestDelay(uuid: String, restDelay: LongRange?) {
        getConfiguration(uuid)["rest-delay"] = restDelay?.toMString()
        uuid pipe this::update
    }

    fun getConfiguration(uuid: String): ink.ptms.adyeshach.taboolib.library.configuration.ConfigurationSection {
        if (!data.contains(uuid)) data[uuid] = mapOf<String, String>()
        return data.getConfigurationSection(uuid)!!
    }

    private fun update(uuid: String) {
        bubbles[uuid] = getBubbles(uuid) ?: return
    }

    private fun fistUpdate(uuid: String) {
        if (bubbles.containsKey(uuid)) return
        update(uuid)
    }


    @Schedule(async = true, period = 1)
    fun tick() {
        for (uuid in visibleEntity.toList()) {
            val bubble = bubbles[uuid] ?: continue
            bubble.tick(uuid)
        }
    }

    fun startOnce(uuid: String) {
        val bubble = bubbles[uuid] ?: return
        if (bubble.config.mode != ChatPopupMode.ONCE) return
        bubble.resetState()
    }

    @SubscribeEvent(priority = EventPriority.MONITOR, ignoreCancelled = true)
    // 仅在可视时渲染聊天气泡
    private fun onVisible(e: AdyeshachEntityVisibleEvent) {
        if (e.entity.isPublic()) {
            val uuid = e.entity.uniqueId
            fistUpdate(uuid)
            if (e.visible) {
                if (visibleEntity.contains(uuid)) return
                visibleEntity.add(uuid)
            } else {
                uuid also visibleEntity::remove also BubblesBundle::clear
            }
        }
    }

    override fun id(): String {
        return "bubbles-chat"
    }
}
